@@ -20,7 +20,21 @@ class surf(object):
         self.c = np.asarray(c) # (multiarray of) constant(s)
 
     value = lambda self, point: self.c + PythonSum(point[k] * v for k, v in self.d.items())
-
+    
+#    def invert(self, ind=None):
+#        ind_is_None = ind is None
+#        C = -self.c if ind_is_None else where(ind, -self.c, self.c)
+#        D = dict((k, 
+#                  (-v if ind_is_None else where(ind, -v, v))) \
+#                  for k, v in self.d.items())
+#        if 'd2' not in self.__dict__:
+#            return surf(D, C)
+#        from boundsurf2 import surf2
+#        D2 = dict((k, 
+#                  (-v if ind_is_None else where(ind, -v, v))) \
+#                  for k, v in self.d2.items())
+#        return surf2(D2, D, C)
+        
 #    resolve = lambda self, domain, cmp: \
 #    self.c + PythonSum(where(cmp(v, 0), domain[k][0], domain[k][1])*v for k, v in self.d.items())
     
@@ -161,6 +175,25 @@ class boundsurf(object):#object is added for Python2 compatibility
         assert self._resolved.shape[0] == 2, 'bug in FD kernel'
         return self._resolved, self.definiteRange
     
+    def invert(self, ind=None):
+        B = self.__class__
+        if ind is None:
+            return B(-self.u, -self.l, self.definiteRange, self.domain)
+#            if ind.dtype != bool:
+#                bool_ind = np.zeros()
+        assert ind.dtype == bool, 'unimplemented yet'
+        ind_same, ind_invert = where(logical_not(ind))[0], where(ind)[0]
+        l1, u1 = self.l.extract(ind_same), self.u.extract(ind_same)
+        l2, u2 = self.l.extract(ind_invert), self.u.extract(ind_invert)
+        b1 = B(l1, u1, False, self.domain)
+        b2 = B(-u2, -l2, False, self.domain)
+        b = boundsurf_join((ind_same, ind_invert), (b1, b2))
+        b.definiteRange = self.definiteRange
+#        l, u = self.u.invert(ind), self.l.invert(ind)
+#        if ind is None:
+        return b
+#        l_unchanged, u_unchanged = self.l.extract()
+    
     def render(self):
         if self.isRendered:
             return
@@ -258,8 +291,16 @@ class boundsurf(object):#object is added for Python2 compatibility
             rr = mul_fixed_interval(self, other)
 
         elif isBoundSurf or isBoundSurf2:
-            if self.level == other.level == 1 and self.u is self.l and other.u is other.l and self.b2equiv(other):
+            sameBounds_1 = self.l is self.u
+            sameBounds_2 = other.l is other.u
+            if self.level == other.level == 1 and sameBounds_1 and sameBounds_2 and self.b2equiv(other):
                 rr = b2mult_direct(self, other)
+            elif ((selfPositive or selfNegative) and sameBounds_1) or ((R2Positive or R2Negative) and sameBounds_2)\
+            and self.level == other.level == 1 and self.b2equiv(other):
+                Self = self if selfPositive else -self
+                Other = other if R2Positive else -other
+                r = b2mult(Self, Other)
+                rr = r if selfPositive == R2Positive else -r
             elif (selfPositive or selfNegative) and (R2Positive or R2Negative):
                 Self = self if selfPositive else -self
                 Other = other if R2Positive else -other
@@ -274,6 +315,7 @@ class boundsurf(object):#object is added for Python2 compatibility
                         r = np.exp(_r)
                         return r if selfPositive == R2Positive else -r[::-1], definiteRange
                     r = _r.exp()
+                    # is definiteRange required here?
                     r.definiteRange = definiteRange
                     
                 rr = r if selfPositive == R2Positive else -r
@@ -673,8 +715,15 @@ def mul_fixed_interval(self, R2):
     R1 = self.resolve()[0]
     selfPositive = all(R1 >= 0)
     selfNegative = all(R1 <= 0)
-        
-    if selfPositive and R2Positive:
+    
+    SelfSameBounds = self.l is self.u
+#    print('0', SelfSameBounds, selfPositive, selfNegative)
+    if 1 and SelfSameBounds and (selfPositive or selfNegative):
+        bound = self.l # equal to self.u
+#        if selfPositive or selfNegative:
+        rr = (bound * R2[0], bound * R2[1]) if selfPositive else (bound * R2[1], bound * R2[0])
+
+    elif selfPositive and R2Positive:
         rr = (self.l * R2[0], self.u * R2[1]) 
     elif selfPositive and R2Negative:
         rr = (self.u * R2[0], self.l * R2[1])
@@ -701,11 +750,33 @@ def mul_fixed_interval(self, R2):
         Tmp = np.vstack((l1*l2, l1*u2, l2*u1, u1*u2))
         tmp_l3 = surf({}, nanmin(Tmp, axis=0))
         tmp_u3 = surf({}, nanmax(Tmp, axis=0))
-        
 
         tmp_l = surf_join((ind_positive, ind_negative, ind_z), (tmp_l1, tmp_l2, tmp_l3))
         tmp_u = surf_join((ind_positive, ind_negative, ind_z), (tmp_u1, tmp_u2, tmp_u3))
 
+#        Inds, L, U = [], [], []
+#        if ind_positive.size:
+#            tmp_l1 = other_lb[ind_positive] * l.extract(ind_positive)
+#            tmp_u1 = other_ub[ind_positive] * u.extract(ind_positive)
+#            Inds.append(ind_positive)
+#            L.append(tmp_l1)
+#            U.append(tmp_u1)
+#        if ind_negative.size:
+#            tmp_l2 = other_ub[ind_negative] * l.extract(ind_negative)
+#            tmp_u2 = other_lb[ind_negative] * u.extract(ind_negative)
+#            Inds.append(ind_negative)
+#            L.append(tmp_l2)
+#            U.append(tmp_u2)
+#        if ind_z.size:
+#            l2, u2 = other_lb[ind_z], other_ub[ind_z]
+#            l1, u1 = lb1[ind_z], ub1[ind_z]
+#            Tmp = np.vstack((l1*l2, l1*u2, l2*u1, u1*u2))
+#            tmp_l3 = surf({}, nanmin(Tmp, axis=0))
+#            tmp_u3 = surf({}, nanmax(Tmp, axis=0))
+#            Inds.append(ind_z)
+#            L.append(tmp_l3)
+#            U.append(tmp_u3)
+
         rr = (tmp_l, tmp_u) if R2Positive else (-tmp_u, -tmp_l)
     elif selfPositive or selfNegative:
         l, u = (self.l, self.u) if selfPositive else (-self.u, -self.l)
